%% Steven R. Woodruff - 2 July 2019 - Curved Square %%

function [InputData] = Geometry_Canopy(InputData)
                        
    numberDivisions = InputData.numberDivisions;
    loadMagnitude = InputData.loadMagnitude;
    plotNodes = InputData.plotNodes;

    syms u
    
    y = [0;
     -2/3*(u-3)^2+1;
     -2/3*(u-3)^2+2;
     -2/3*(u-3)^2+3;
     -2/3*(u-3)^2+4;
     -2/3*(u-3)^2+5;
     -2/3*(u-3)^2+6];
 
    poly = [-2/3 4 -5;
            -2/3 4 -4;
            -2/3 4 -3;
            -2/3 4 -2;
            -2/3 4 -1;
            -2/3 4 0];
 
    root = sort(roots(poly(1,:)));
    x1 = root(1);
    x2 = root(2);
    dx = (x2-x1)/numberDivisions;
    
                                    
%% Calculate the mesh nodes
    numberNodes = 6*(numberDivisions+1);
    numberNodesPerRow = numberDivisions+1;
    numberCurves = size(poly,1);
    Nodes = zeros(numberNodes,3);

    x_i = x1+dx;
    row = 1;
    
    for i = 1:numberNodes
        if mod(i-1,numberNodesPerRow) == 0
            row = row + 1;
            root = sort(roots(poly(row-1,:)));
            x1 = root(1);
            x2 = root(2);
            dx = (x2-x1)/numberDivisions;
            x_i = x1;
        end
        Nodes(i,:) = [x_i subs(y(row),u,x_i) 0];
        x_i = x_i + dx;
    end
    
    nodesFold = (numberNodesPerRow+1):1:(5*numberNodesPerRow);
    nodesValley = [(2*numberNodesPerRow+1):1:3*numberNodesPerRow (4*numberNodesPerRow+1):1:(5*numberNodesPerRow)]';
          
%% Boundary conditions
    Tip = (numberCurves-1)*numberNodesPerRow + ceil(numberNodesPerRow/2);

    supports = [1 1 1 1;
                numberNodesPerRow 0 1 1;
                Tip 0 0 1];

    loads = [2 0 0 0];

             
%% Calculate the nodal connections (panels)
    
    numberPanels = (numberDivisions*5)*2; 
    N1 = zeros(numberPanels,1);
    N2 = N1; N3 = N1; N4 = N1;
    Add = 0;
    Sign = 1;
    for i = 1:numberPanels/2
        N1(i) = i + Add;
        N2(i) = N1(i) + numberNodesPerRow;
        N3(i) = N1(i) + numberNodesPerRow + 1;
        N4(i) = N1(i) + 1;    
        if mod(i,numberNodesPerRow-1) == 0
            Add = Add + 1;
        end
        if Sign == 1
           Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
           Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
           if mod(i,numberNodesPerRow-1) == 0
               Sign = 1;
           else
               Sign = -1;
           end
        else 
           Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
           Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
           if mod(i,numberNodesPerRow-1) == 0
               Sign = -1;
           else
               Sign = 1;
           end
        end    
    end; clear i Add
    clear N1 N2 N3 N4
    
             
   %% Plot the nodes and mesh
    
    InputData.nodes = Nodes;
    InputData.panels = Panels;
    InputData.supports = supports;
    InputData.loads = loads;
    InputData.foldNodes = nodesFold;
    InputData.valleyNodes = nodesValley;
    
    No_Panels = size(Panels,2);
    
       if strcmpi(InputData.plotNodes,'yes')
        figure()
            for i = 1:No_Panels
                line([InputData.nodes(InputData.panels{i}(1),1), InputData.nodes(InputData.panels{i}(2),1)], [InputData.nodes(InputData.panels{i}(1),2), InputData.nodes(InputData.panels{i}(2),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(2),1), InputData.nodes(InputData.panels{i}(3),1)], [InputData.nodes(InputData.panels{i}(2),2), InputData.nodes(InputData.panels{i}(3),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(3),1), InputData.nodes(InputData.panels{i}(1),1)], [InputData.nodes(InputData.panels{i}(3),2), InputData.nodes(InputData.panels{i}(1),2)],'Color',[140 140 140]./255); hold on
            end; clear i
            scatter(InputData.nodes(:,1),InputData.nodes(:,2),'k.'); hold on
            for i = 1:size(InputData.supports,1)
                if InputData.supports(i,2) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),80,'^','filled','MarkerFaceColor',[65 69 136]./255); hold on
                end
                if InputData.supports(i,3) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),60,'>','filled','MarkerFaceColor',[43 120 142]./255); hold on
                end
                if InputData.supports(i,4) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),50,'v','filled','MarkerFaceColor',[31 168 132]./255); hold on
                end
            end
            for i = 1:length(InputData.foldNodes)-1
                if ~(mod(i,InputData.numberDivisions+1) == 0)
                    if ~isempty(find(InputData.foldNodes(i) == InputData.valleyNodes,1))
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[67 24 83]./255,'LineWidth',2); hold on
                    else
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[31 168 132]./255,'LineWidth',2); hold on
                    end
                end
            end                   
            scatter(InputData.nodes(InputData.loads(:,1),1),InputData.nodes(InputData.loads(:,1),2),'o','filled','MarkerFaceColor',[129 196 85]./255); hold on
            axis equal
            axis off
    end
end
  
            
    
   
    
    