%% Steven R. Woodruff - 2 July 2019 - Curved Square %%

function [InputData] = Geometry_CurvedSquare(InputData)
                        
    number_divisions = InputData.numberDivisions;
    Load_Magnitude = InputData.loadMagnitude;
    Plot_Nodes = InputData.plotNodes;

    syms u
    
    y1 = -2/9*(u-3)^2 + 2;
    y2 = 2/9*(u-3)^2 + 4;
    
    W = 6;
    H = 6;
                                    
%% Calculate the mesh nodes
    No_Nodes = 4*(number_divisions+1);
    No_Nodes_Row = number_divisions+1;
    Nodes = zeros(No_Nodes,3);

    x_i = 0;
    
    for i = 1:No_Nodes
        if i <= No_Nodes_Row
            Nodes(i,:) = [x_i 0 0];
            x_i = x_i + W/number_divisions;
        elseif i > No_Nodes_Row*3
            if i == 3*No_Nodes_Row+1
                x_i = 0;
            end
            Nodes(i,:) = [x_i H 0];
            x_i = x_i + W/number_divisions;
        elseif i > No_Nodes_Row && i <= 2*No_Nodes_Row
            if i == No_Nodes_Row+1
                x_i = 0;
            end
            Nodes(i,:) = [x_i subs(y1,u,x_i) 0];
            x_i = x_i + W/number_divisions;
        else
            if i == 2*No_Nodes_Row+1
                x_i = 0;
            end
            Nodes(i,:) = [x_i subs(y2,u,x_i) 0];
            x_i = x_i + W/number_divisions;
        end
    end
    
    Nodes_Fold = [1 (No_Nodes_Row+2):1:(3*No_Nodes_Row)];
    Nodes_Valley = [];
    
%     figure()
%         scatter(Nodes(:,1), Nodes(:,2))
      
%% Boundary conditions
    Supports = [1 1 1 1;
                No_Nodes_Row 0 1 1;
                size(Nodes,1) 0 0 1];

    Loads = [2*No_Nodes_Row+1 0 0 0];

             
%% Calculate the nodal connections (panels)
    
    No_Panels = (number_divisions*3)*2-4; 
    N1 = zeros(No_Panels,1);
    N2 = N1; N3 = N1; N4 = N1;
    Add = 0;
    Sign = 1;
    panel_count = 0;
    for i = 1:(No_Panels+4)/2       
        N1(i) = i + Add;
        N2(i) = N1(i) + No_Nodes_Row;
        N3(i) = N1(i) + No_Nodes_Row + 1;
        N4(i) = N1(i) + 1;   
        if i == 1
            panel_count = panel_count+1;
            Panels{panel_count} = [N1(i) N3(i) N4(i)];
        elseif i == number_divisions
            panel_count = panel_count+1;
            Panels{panel_count} = [N1(i) N2(i) N4(i)];
            Add = Add + 1;
        elseif i == 2*number_divisions+1
            panel_count = panel_count+1;
            Panels{panel_count} = [N2(i) N3(i) N4(i)];
        elseif i == 3*number_divisions
            panel_count = panel_count+1;
            Panels{panel_count} = [N1(i) N2(i) N3(i)];
        elseif Sign == 1
            panel_count = panel_count+1;
            Panels{panel_count} = [N1(i) N2(i) N3(i)];
            panel_count = panel_count+1;
            Panels{panel_count} = [N1(i) N3(i) N4(i)];
           if mod(i,No_Nodes_Row-1) == 0
               Sign = 1;
           else
               Sign = -1;
           end
        else 
           panel_count = panel_count+1;
           Panels{panel_count} = [N1(i) N2(i) N4(i)];
           panel_count = panel_count+1;
           Panels{panel_count} = [N2(i) N3(i) N4(i)];
           if mod(i,No_Nodes_Row-1) == 0
               Sign = -1;
           else
               Sign = 1;
           end
        end    
        if i == 2*number_divisions
            Add = Add + 1;
        end
%         Plot_Flat(Nodes,Panels,Supports,Loads,Nodes_Fold,Nodes_Valley,number_divisions);
    end; clear i Add
    clear N1 N2 N3 N4
    
             
   %% Plot the nodes and mesh    
    InputData.nodes = Nodes;
    InputData.panels = Panels;
    InputData.supports = Supports;
    InputData.loads = Loads;
    InputData.foldNodes = Nodes_Fold;
    InputData.valleyNodes = Nodes_Valley;
    
    if strcmpi(InputData.plotNodes,'yes')
        figure()
            for i = 1:No_Panels
                line([InputData.nodes(InputData.panels{i}(1),1), InputData.nodes(InputData.panels{i}(2),1)], [InputData.nodes(InputData.panels{i}(1),2), InputData.nodes(InputData.panels{i}(2),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(2),1), InputData.nodes(InputData.panels{i}(3),1)], [InputData.nodes(InputData.panels{i}(2),2), InputData.nodes(InputData.panels{i}(3),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(3),1), InputData.nodes(InputData.panels{i}(1),1)], [InputData.nodes(InputData.panels{i}(3),2), InputData.nodes(InputData.panels{i}(1),2)],'Color',[140 140 140]./255); hold on
            end; clear i
            scatter(InputData.nodes(:,1),InputData.nodes(:,2),'k.'); hold on
            for i = 1:size(InputData.supports,1)
                if InputData.supports(i,2) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),80,'^','filled','MarkerFaceColor',[65 69 136]./255); hold on
                end
                if InputData.supports(i,3) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),60,'>','filled','MarkerFaceColor',[43 120 142]./255); hold on
                end
                if InputData.supports(i,4) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),50,'v','filled','MarkerFaceColor',[31 168 132]./255); hold on
                end
            end
            for i = 1:length(InputData.foldNodes)-1
                if ~(mod(i,InputData.numberDivisions+1) == 0)
                    if ~isempty(find(InputData.foldNodes(i) == InputData.valleyNodes,1))
                        line([InputData.nodes(InputData.foldNodes(i),1), ...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[67 24 83]./255,'LineWidth',2); hold on
                    else
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[31 168 132]./255,'LineWidth',2); hold on
                    end
                end
            end                   
            scatter(InputData.nodes(InputData.loads(:,1),1),InputData.nodes(InputData.loads(:,1),2),'o','filled','MarkerFaceColor',[129 196 85]./255); hold on
            axis equal
            axis off
    end
end
  
            
    
   
    
    