%% Steven R. Woodruff - 25 July 2019 - Cut Annulus Mesher %%

function [InputData] = Geometry_CutAnnulus(InputData)

    number_divisions = InputData.numberDivisions;
    Load_Magnitude = InputData.loadMagnitude;
    Plot_Nodes = InputData.plotNodes;
                                                                                        
    syms u % The parametric value of the curves
    
    r = 100;
    w = 5;
    theta = pi/4; 
    MeshConfig = 1;

    %% 4. Cut creased annulus (blam = 0.005)
    Curves = [(r+w)*cos(u) (r+w)*sin(u);
              r*cos(u) r*sin(u);
              (r-w)*cos(u) (r-w)*sin(u)];
    Curve_Rollers = [1; 0; 1];
    Val_Flds = [0; 0 ; 0];
    u_min = 0;
    u_max = theta;
             
%% Calculate curve lengths
        
    No_Curves = size(Curves,1);
    
    du = (u_max-u_min)/number_divisions; % Parametric span length of each bar
    
%% Calculate the mesh nodes
    
    Nodes = [];
    Nodes_Fold = [];
    Nodes_Roller = [];
    Nodes_Valley = [];
    for i = 1:No_Curves
        for j = 1:(number_divisions+1)
            Nodes = [Nodes; subs(Curves(i,:),u,du*(j-1)+u_min)];
            if i ~= 1 && i ~= No_Curves
                Nodes_Fold = [Nodes_Fold; size(Nodes,1)];
            end 
            if Curve_Rollers(i) == 1
                Nodes_Roller = [Nodes_Roller; size(Nodes,1)];
            end
            if Val_Flds(i) == 1
                Nodes_Valley = [Nodes_Valley; size(Nodes,1)];
            end
        end; clear j
    end; clear i
    Nodes = double(Nodes);
    Nodes = [Nodes zeros(size(Nodes,1),1)];
    Nodes_Fold = double(Nodes_Fold)';
    Nodes_Valley = [];
    No_Nodes_Row = number_divisions+1;
        
%% Calculate the nodal connections (panels) 
    
    No_Elements = number_divisions*(No_Curves-1);
    
    N1 = zeros(No_Elements,1);
    N2 = N1; N3 = N1; N4 = N1;
    Add = 0;
    for i = 1:No_Elements
        N1(i) = i + Add;
        N2(i) = N1(i) + number_divisions+1;
        N3(i) = N1(i) + number_divisions+2;
        N4(i) = N1(i) + 1;
        if mod(i,number_divisions) == 0
            Add = Add + 1;
        end
        if i<No_Elements/2+1  
            if MeshConfig == 1
                if mod(i,2)
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
                    Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
                else      
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
                    Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
                end    
            else
                if mod(i,2)
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
                    Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
                else      
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
                    Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
                end  
            end
        else
            if MeshConfig == 1
                if mod(i,2)
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
                    Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)]; 
                else     
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
                    Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
                end   
            else
                if mod(i,2)
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
                    Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
                else     
                    Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
                    Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)]; 
                end   
            end
        end
    end; clear i Add N1 N2 N3 N4
    
   %% Boundary conditions
   
%    Nodes_Edges = [2:1:No_Nodes_Row 2*(No_Nodes_Row)+2:1:3*No_Nodes_Row];
    
   Supports = [1 1 1 1;
               2*(No_Nodes_Row)+1 0 1 1;
               (No_Nodes_Row) 0 0 1];
%                Nodes_Edges' zeros(length(Nodes_Edges),1) zeros(length(Nodes_Edges),1) ones(length(Nodes_Edges),1)];
          

    Loads = [1 0 0 0];

   %% Plot the nodes and mesh     
    InputData.nodes = Nodes;
    InputData.panels = Panels;
    InputData.supports = Supports;
    InputData.loads = Loads;
    InputData.foldNodes = Nodes_Fold;
    InputData.valleyNodes = Nodes_Valley;
    
    No_Panels = size(Panels,2);
    
    if strcmpi(InputData.plotNodes,'yes')
        figure()
            for i = 1:No_Panels
                line([InputData.nodes(InputData.panels{i}(1),1), InputData.nodes(InputData.panels{i}(2),1)], [InputData.nodes(InputData.panels{i}(1),2), InputData.nodes(InputData.panels{i}(2),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(2),1), InputData.nodes(InputData.panels{i}(3),1)], [InputData.nodes(InputData.panels{i}(2),2), InputData.nodes(InputData.panels{i}(3),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(3),1), InputData.nodes(InputData.panels{i}(1),1)], [InputData.nodes(InputData.panels{i}(3),2), InputData.nodes(InputData.panels{i}(1),2)],'Color',[140 140 140]./255); hold on
            end; clear i
            scatter(InputData.nodes(:,1),InputData.nodes(:,2),'k.'); hold on
            for i = 1:size(InputData.supports,1)
                if InputData.supports(i,2) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),80,'^','filled','MarkerFaceColor',[65 69 136]./255); hold on
                end
                if InputData.supports(i,3) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),60,'>','filled','MarkerFaceColor',[43 120 142]./255); hold on
                end
                if InputData.supports(i,4) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),50,'v','filled','MarkerFaceColor',[31 168 132]./255); hold on
                end
            end
            for i = 1:length(InputData.foldNodes)-1
                if ~(mod(i,InputData.numberDivisions+1) == 0)
                    if ~isempty(find(InputData.foldNodes(i) == InputData.valleyNodes,1))
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[67 24 83]./255,'LineWidth',2); hold on
                    else
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[31 168 132]./255,'LineWidth',2); hold on
                    end
                end
            end                   
            scatter(InputData.nodes(InputData.loads(:,1),1),InputData.nodes(InputData.loads(:,1),2),'o','filled','MarkerFaceColor',[129 196 85]./255); hold on
            axis equal
            axis off
    end
    
end
    
  
            
    
   
    
    