%% Steven R. Woodruff - 1 Nov 2018 - Unit Mesher %%

function [InputData] = Geometry_FoldedStrip(InputData)
    No_Units_x = InputData.numberDivisions;
    Unit_Scale_x = 10/No_Units_x;
    Unit_Scale_y = 1;
    Load_Magnitude = InputData.loadMagnitude;
    Test_Type = 'Stretching';
    Nodes_Valley = [];
             
%% Calculate the mesh nodes
    No_Nodes = 3*(No_Units_x+1);
    No_Nodes_Row = No_Nodes/3;
    Nodes = zeros(No_Nodes,3);
    
    for i = 1:No_Nodes
        if i == 1
            Nodes(i,:) = [0 0 0];
        elseif mod(i,No_Nodes_Row) == 1
            Nodes(i,:) = [0 floor(i/No_Nodes_Row)*Unit_Scale_y/2 0];
        else
            Nodes(i,:) = Nodes(i-1,:) + [Unit_Scale_x/2 0 0];
        end
    end
    
    Nodes_Fold = (No_Nodes_Row+1):1:(2*No_Nodes_Row);
             
%% Calculate the nodal connections (panels)
    
    No_Panels = 4*No_Units_x;      
    N1 = zeros(No_Panels,1);
    N2 = N1; N3 = N1; N4 = N1;
    Add = 0;
    Sign = 1;
    for i = 1:No_Panels/2
        N1(i) = i + Add;
        N2(i) = N1(i) + No_Nodes_Row;
        N3(i) = N1(i) + No_Nodes_Row + 1;
        N4(i) = N1(i) + 1;    
        if mod(i,No_Nodes_Row-1) == 0
            Add = Add + 1;
        end
        if Sign == 1
           Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
           Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
           if mod(i,No_Nodes_Row-1) == 0
               Sign = 1;
           else
               Sign = -1;
           end
        else 
           Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
           Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
           if mod(i,No_Nodes_Row-1) == 0
               Sign = -1;
           else
               Sign = 1;
           end
        end    
    end; clear i Add
    clear N1 N2 N3 N4
    
    
%% Boundary conditions

    if strcmp(Test_Type,'Bending')
        Supports = [1 1 1 1;
                    (No_Nodes_Row+1) 1 1 1;
                    (2*No_Nodes_Row+1) 1 1 1;
                    No_Nodes_Row 0 0 1;
                    2*No_Nodes_Row 0 0 1;
                    3*No_Nodes_Row 0 0 1];
        Loads = [2 0 0 -Load_Magnitude;
                 (No_Nodes_Row-1) 0 0 -Load_Magnitude;
                 (No_Nodes_Row+2) 0 0 -Load_Magnitude;
                 (2*No_Nodes_Row-1) 0 0 -Load_Magnitude;
                 (2*No_Nodes_Row+2) 0 0 -Load_Magnitude;
                 (3*No_Nodes_Row-1) 0 0 -Load_Magnitude];
    elseif strcmp(Test_Type,'Folding')
        Supports = [1 1 1 1;
                    No_Nodes_Row 0 0 1;
                    2*No_Nodes_Row+1 1 0 1];
        Loads = [No_Nodes_Row+1 0 0 Load_Magnitude];
    elseif strcmp(Test_Type,'Twisting')
        Supports = [(No_Nodes_Row+1) 1 1 1;
                    2*No_Nodes_Row 1 1 1;
                    (No_Nodes_Row/2+0.5) 0 0 1;
                    (3*No_Nodes_Row-No_Nodes_Row/2+0.5) 0 0 1];
        Loads = [1 0 0 -Load_Magnitude;
                 No_Nodes_Row 0 0 Load_Magnitude;
                 (2*No_Nodes_Row+1) 0 0 Load_Magnitude;
                 3*No_Nodes_Row 0 0 -Load_Magnitude];
    elseif strcmp(Test_Type,'Stretching')
        Supports = [1 1 1 1;
                    (No_Nodes_Row+1) 1 0 1;
%                     No_Nodes_Row 0 0 1;
                    (2*No_Nodes_Row+1) 1 0 1];
        if strcmpi(InputData.loadType, 'Force')
            Loads = [No_Nodes_Row Load_Magnitude/4 0 0;
                     2*No_Nodes_Row Load_Magnitude/2 0 0;
                     3*No_Nodes_Row Load_Magnitude/4 0 0];
        else
            Loads = [No_Nodes_Row Load_Magnitude 0 0;
                     2*No_Nodes_Row Load_Magnitude 0 0;
                     3*No_Nodes_Row Load_Magnitude 0 0];
        end
    elseif strcmp(Test_Type,'Shearing')
        Supports = [1 1 1 1;
                    (No_Nodes_Row+1) 1 1 1;
                    (2*No_Nodes_Row+1) 1 1 1];
        Loads = [No_Nodes_Row 0 -Load_Magnitude 0;
                 2*No_Nodes_Row 0 -Load_Magnitude 0;
                 3*No_Nodes_Row 0 -Load_Magnitude 0];
    else
        fprintf('\n\nError: Test_Type invalid\n\n')
        Supports = [];
        Loads = [];
        return
    end
          
   %% Plot the nodes and mesh

InputData.nodes = Nodes;
    InputData.panels = Panels;
    InputData.supports = Supports;
    InputData.loads = Loads;
    InputData.foldNodes = Nodes_Fold;
    InputData.valleyNodes = Nodes_Valley;
    
    if strcmpi(InputData.plotNodes,'yes')
        figure()
            for i = 1:No_Panels
                line([InputData.nodes(InputData.panels{i}(1),1), InputData.nodes(InputData.panels{i}(2),1)], [InputData.nodes(InputData.panels{i}(1),2), InputData.nodes(InputData.panels{i}(2),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(2),1), InputData.nodes(InputData.panels{i}(3),1)], [InputData.nodes(InputData.panels{i}(2),2), InputData.nodes(InputData.panels{i}(3),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(3),1), InputData.nodes(InputData.panels{i}(1),1)], [InputData.nodes(InputData.panels{i}(3),2), InputData.nodes(InputData.panels{i}(1),2)],'Color',[140 140 140]./255); hold on
            end; clear i
            scatter(InputData.nodes(:,1),InputData.nodes(:,2),'k.'); hold on
            for i = 1:size(InputData.supports,1)
                if InputData.supports(i,2) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),80,'^','filled','MarkerFaceColor',[65 69 136]./255); hold on
                end
                if InputData.supports(i,3) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),60,'>','filled','MarkerFaceColor',[43 120 142]./255); hold on
                end
                if InputData.supports(i,4) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),50,'v','filled','MarkerFaceColor',[31 168 132]./255); hold on
                end
            end
            for i = 1:length(InputData.foldNodes)-1
                if ~(mod(i,InputData.numberDivisions+1) == 0)
                    if ~isempty(find(InputData.foldNodes(i) == InputData.valleyNodes,1))
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[67 24 83]./255,'LineWidth',2); hold on
                    else
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[31 168 132]./255,'LineWidth',2); hold on
                    end
                end
            end                   
            scatter(InputData.nodes(InputData.loads(:,1),1),InputData.nodes(InputData.loads(:,1),2),'o','filled','MarkerFaceColor',[129 196 85]./255); hold on
            axis equal
            axis off
    end
    
  
            
    
   
    
    