%% Steven R. Woodruff - 1 July 2019 - Full Annulus Mesher %%

function [InputData] = Geometry_FullAnnulus(InputData)
                       
    number_divisions = InputData.numberDivisions;
    Load_Magnitude = InputData.loadMagnitude;
    Plot_Nodes = InputData.plotNodes;

    r_fold = 100;
    w = 10;
    
    Nodes_Valley = [];
             
%% Calculate the mesh nodes
    No_Nodes = 3*number_divisions;
    No_Nodes_Row = number_divisions;
    Nodes = zeros(No_Nodes,3);
    
    for i = 1:No_Nodes
        if i == 1
            r_i = r_fold - w;
            t_i = 0;
        elseif mod(i,No_Nodes_Row) == 1
            r_i = r_i + w;
            t_i = 0;
        else
            t_i = t_i + 2*pi/number_divisions;
        end
        Nodes(i,:) = [r_i*cos(t_i) r_i*sin(t_i) 0];
    end
    
    Nodes_Fold = [(No_Nodes_Row+1):1:(2*No_Nodes_Row) No_Nodes_Row+1];
             
%% Calculate the nodal connections (panels)
    
    No_Panels = 4*number_divisions;      
    N1 = zeros(2*No_Panels,1);
    N2 = N1; N3 = N1; N4 = N1;
    Add = 0;
    Sign = 1;
    Count = 0;
    for i = 1:No_Panels/2
        if mod(i,number_divisions) == 0
            N1(i) = i;
            N2(i) = N1(i) + No_Nodes_Row;
            if Count == 0
                N3(i) = 1 + No_Nodes_Row;
                N4(i) = 1;
                Count = 1;
            else
                N3(i) = 2*No_Nodes_Row + 1;
                N4(i) = No_Nodes_Row + 1;
            end
        else
            N1(i) = i;
            N2(i) = N1(i) + No_Nodes_Row;
            N3(i) = N1(i) + No_Nodes_Row + 1;
            N4(i) = N1(i) + 1;
        end
        if i == number_divisions+1
           Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
           Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
           Sign = 1;
        elseif Sign == 1
           Panels{(i-1)*2+1} = [N1(i) N2(i) N3(i)];
           Panels{(i-1)*2+2} = [N1(i) N3(i) N4(i)];
           Sign = -1;
        else 
           Panels{(i-1)*2+1} = [N1(i) N2(i) N4(i)];
           Panels{(i-1)*2+2} = [N2(i) N3(i) N4(i)];
           Sign = 1;
        end    
    end; clear i Add
    clear N1 N2 N3 N4
    
    
%% Boundary conditions
    Supports = [1 1 1 1;
                number_divisions/2+1 0 1 1;
                number_divisions/4+1 0 0 1];
    Loads = [3*number_divisions/4+1 1e-12 0 0];                
          
   %% Plot the nodes and mesh
    
    InputData.nodes = Nodes;
    InputData.panels = Panels;
    InputData.supports = Supports;
    InputData.loads = Loads;
    InputData.foldNodes = Nodes_Fold;
    InputData.valleyNodes = Nodes_Valley;
    
    if strcmpi(InputData.plotNodes,'yes')
        figure()
            for i = 1:No_Panels
                line([InputData.nodes(InputData.panels{i}(1),1), InputData.nodes(InputData.panels{i}(2),1)], [InputData.nodes(InputData.panels{i}(1),2), InputData.nodes(InputData.panels{i}(2),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(2),1), InputData.nodes(InputData.panels{i}(3),1)], [InputData.nodes(InputData.panels{i}(2),2), InputData.nodes(InputData.panels{i}(3),2)],'Color',[140 140 140]./255); hold on
                line([InputData.nodes(InputData.panels{i}(3),1), InputData.nodes(InputData.panels{i}(1),1)], [InputData.nodes(InputData.panels{i}(3),2), InputData.nodes(InputData.panels{i}(1),2)],'Color',[140 140 140]./255); hold on
            end; clear i
            scatter(InputData.nodes(:,1),InputData.nodes(:,2),'k.'); hold on
            for i = 1:size(InputData.supports,1)
                if InputData.supports(i,2) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),80,'^','filled','MarkerFaceColor',[65 69 136]./255); hold on
                end
                if InputData.supports(i,3) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),60,'>','filled','MarkerFaceColor',[43 120 142]./255); hold on
                end
                if InputData.supports(i,4) == 1
                    scatter(InputData.nodes(InputData.supports(i,1),1),InputData.nodes(InputData.supports(i,1),2),50,'v','filled','MarkerFaceColor',[31 168 132]./255); hold on
                end
            end
            for i = 1:length(InputData.foldNodes)-1
                if ~(mod(i,InputData.numberDivisions+1) == 0)
                    if ~isempty(find(InputData.foldNodes(i) == InputData.valleyNodes,1))
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[67 24 83]./255,'LineWidth',2); hold on
                    else
                        line([InputData.nodes(InputData.foldNodes(i),1),...
                             InputData.nodes(InputData.foldNodes(i+1),1)],...
                             [InputData.nodes(InputData.foldNodes(i),2),...
                             InputData.nodes(InputData.foldNodes(i+1),2)],...
                             'Color',[31 168 132]./255,'LineWidth',2); hold on
                    end
                end
            end                   
            scatter(InputData.nodes(InputData.loads(:,1),1),InputData.nodes(InputData.loads(:,1),2),'o','filled','MarkerFaceColor',[129 196 85]./255); hold on
            axis equal
            axis off
    end
    
end